### 普通字符

表达式 | 描述
:--|:--
`.` 或 `[^\n\r]` | 任何字符 *不包括* 换行或回车
`[A-Za-z]` | 字母
`[a-z]` | 小写字母
`[A-Z]` | 大写字母
`\d` 或 `[0-9]` | 数字
`\D` 或 `[^0-9]` | 非数字
`_` | 下划线
`\w` 或 `[A-Za-z0-9_]` | 字母、数字或下划线
`\W` 或 `[^A-Za-z0-9_]` | `\w` 的反义
`\S` | `\s` 的反义

### 空白字符

表达式 | 描述
:--|:--
` ` | 空格
`\t` | 制表符
`\n` | 换行
`\r` | 回车
`\s` | 空格、制表符、换行或回车

### 字符集

表达式 | 描述
:--|:--
`[xyz]` | `x`、`y` 或 `z` 中的任意一个
`[^xyz]` | 既不是 `x`、`y` 也不是 `z`
`[1-3]` | `1`、`2` 或 `3` 中的任意一个
`[^1-3]` | 既不是 `1`、`2` 也不是 `3`

- 将字符集视为对方括号内单个字符的 `OR` 操作。
- 在开括号 `[` 后使用 `^` 来“否定”字符集。
- 在字符集中，`.` 表示字面上的句号。

### 需要转义的字符

#### 在字符集外

表达式 | 描述
:--|:--
`\.` | 句号
`\^` | 脱字符
`\$` | 美元符号
`\|` | 竖线
`\\` | 反斜杠
`\/` | 斜线
`\(` | 开括号
`\)` | 闭括号
`\[` | 开方括号
`\]` | 闭方括号
`\{` | 开花括号
`\}` | 闭花括号

#### 在字符集内

表达式 | 描述
:--|:--
`\\` | 反斜杠
`\]` | 闭方括号

- 只有当 `^` 紧接在字符集的开括号 `[` 后面时才需要转义。
- 只有当 `-` 出现在两个字母或两个数字之间时才需要转义。

### 量词

表达式 | 描述
:--|:--
`{2}` | 精确 2 次
`{2,}` | 至少 2 次
`{2,7}` | 至少 2 次但不超过 7 次
`*` | 0 次或更多
`+` | 1 次或更多
`?` | 精确 0 次或 1 次

- 量词位于待量词化的表达式 *之后*。

### 边界

表达式 | 描述
:--|:--
`^` | 字符串开始
`$` | 字符串结束
`\b` | 单词边界

- 单词边界匹配的工作原理：
    - 在字符串开始时，如果第一个字符是 `\w`。
    - 在字符串中两个相邻字符之间，如果第一个字符是 `\w`，而第二个字符是 `\W`。
    - 在字符串结束时，如果最后一个字符是 `\w`。

### 匹配

表达式 | 描述
:--|:--
`foo\|bar` | 匹配 `foo` 或 `bar`
`foo(?=bar)` | 如果 `foo` 在 `bar` 前面则匹配 `foo`
`foo(?!bar)` | 如果 `foo` *不* 在 `bar` 前面则匹配 `foo`
`(?<=bar)foo` | 如果 `foo` 在 `bar` 后面则匹配 `foo`
`(?<!bar)foo` | 如果 `foo` *不* 在 `bar` 后面则匹配 `foo`

### 分组与捕获

表达式 | 描述
:--|:--
`(foo)` | 捕获组；匹配并捕获 `foo`
`(?:foo)` | 非捕获组；匹配 `foo` 但 *不* 捕获 `foo`
`(foo)bar\1` | `\1` 是对第 1 个捕获组的回参考；匹配 `foobarfoo`

- 捕获组仅在以下方法中相关：
    - `string.match(regexp)`
    - `string.matchAll(regexp)`
    - `string.replace(regexp, callback)`
- `\N` 是对第 `N` 个捕获组的回参考。捕获组的编号从 1 开始。

## 参考文献和工具

- [MDN](https://developer.mozilla.org/en-US/docs/Web/JavaScript/Guide/Regular_Expressions)
- [RegExplained](https://leaverou.github.io/regexplained/)
